@@ -18,19 +18,18 @@
 #include <stdlib.h>
 
 #ifdef USE_MAEMO
-#include <hildon/hildon-program.h>
 #include <libosso.h>
+#endif
 
-#if (MAEMO_VERSION_MAJOR == 5)
+#include "config.h"
+#include "menu.h"
+#include "track.h"
+
+#ifdef MAEMO5
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 #endif
 
-#endif
-
-#include <gtk/gtk.h>
-#include "config.h"
-
 #include <locale.h>
 #include <libintl.h>
 
@@ -39,6 +38,7 @@
 
 #include "osm-gps-map.h"
 #include "osm-gps-map-osd-classic.h"
+#include "converter.h"
 
 #include "gps.h"
 
@@ -152,8 +152,19 @@ cb_map_gps(osd_button_t but, void *data) {
   }
 }
 
-void gps_callback(int status, struct gps_fix_t *fix, void *data) {
+#ifdef USE_MAEMO
+/* http://mathforum.org/library/drmath/view/51722.html */
+static float get_distance(coord_t *p1, coord_t *p2) {
+  return acos(cos(p1->rlat) * cos(p2->rlat) * cos(p2->rlon - p1->rlon) +
+	      sin(p1->rlat) * sin(p2->rlat)) * 6378137.0;
+}
+#endif
+
+static void gps_callback(int status, struct gps_fix_t *fix, void *data) {
   OsmGpsMap *map = OSM_GPS_MAP(data);
+#ifdef USE_MAEMO
+  GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(map));
+#endif
 
   int gps_status = 
     (int)g_object_get_data(G_OBJECT(map), "gps_status"); 
@@ -182,13 +193,55 @@ void gps_callback(int status, struct gps_fix_t *fix, void *data) {
     g_object_set_data(G_OBJECT(map), "gps_fix", NULL); 
     osm_gps_map_clear_gps(map);
   }
+
+#ifdef USE_MAEMO
+  if(status && toplevel) {
+    /* check gps position every 10 seconds and trigger */
+    /* screen saver if useful */
+    time_t last = (time_t)g_object_get_data(G_OBJECT(map), "gps_timer"); 
+    time_t now = time(NULL);
+    if(now-last > 10) {
+      /* get last saved position */
+      coord_t *last_pos = g_object_get_data(G_OBJECT(map), "gps_pos"); 
+      coord_t cur = { .rlat = deg2rad(fix->latitude),
+		      .rlon = deg2rad(fix->longitude) };
+
+      if(last_pos) {
+	/* consider everything under 3 kph (~2 mph) to be static and */
+	/* disable (trigger) screensaver only above this. 3kph = ~8m/10sec */
+
+	/* compare with reported position */
+	if(get_distance(last_pos, &cur) > 8) {
+	  printf("trigger screen saver after %d ...\n", (int)(now-last));
+
+	  osso_context_t *osso_context =  
+	    g_object_get_data(G_OBJECT(toplevel), "osso-context");
+	  g_assert(osso_context);
+
+	  osso_display_blanking_pause(osso_context);
+
+	} else
+	  printf("too slow, no trigger\n");
+      } else
+	last_pos = g_new0(coord_t, 1);
+      
+      /* make current position the old one */
+      last_pos->rlat = cur.rlat;
+      last_pos->rlon = cur.rlon;
+      
+      g_object_set_data(G_OBJECT(map), "gps_timer", (gpointer)now); 
+      g_object_set_data(G_OBJECT(map), "gps_pos", last_pos); 
+    }
+  }
+#endif
+
 }
 
 static GtkWidget *map_new(void) {
   /* It is recommanded that all applications share these same */
   /* map path, so data is only cached once. The path should be: */
   /* ~/.osm-gps-map on standard PC     (users home) */
-  /* /home/user/.osm-gps-map on Maemo5 (ext3 on internal card) */
+  /* /home/user/MyDocs/.maps on Maemo5 (vfat on internal card) */
   /* /media/mmc2/osm-gps-map on Maemo4 (vfat on internal card) */
 #if !defined(USE_MAEMO)
   char *p = getenv("HOME");
@@ -196,12 +249,24 @@ static GtkWidget *map_new(void) {
   char *path = g_strdup_printf("%s/.osm-gps-map", p);
 #else
 #if MAEMO_VERSION_MAJOR == 5
-  char *path = g_strdup("/home/user/.osm-gps-map");
+  /* early maep releases used the ext3 for the tile cache */
+#define OLD_PATH "/home/user/.osm-gps-map"
+#define NEW_PATH "/home/user/MyDocs/.maps"
+  char *path = NULL;
+
+  /* check if the old path exists, and is not a symlink, then use it */
+  if( g_file_test(OLD_PATH, G_FILE_TEST_IS_DIR) &&
+     !g_file_test(OLD_PATH, G_FILE_TEST_IS_SYMLINK))
+    path = g_strdup(OLD_PATH);
+  else
+    path = g_strdup(NEW_PATH);
 #else
   char *path = g_strdup("/media/mmc2/osm-gps-map");
 #endif
 #endif
 
+  printf("Storing tile cache at %s\n", path);
+
   const char *proxy = get_proxy_uri();
 
   GConfClient *gconf_client = gconf_client_get_default();
@@ -229,13 +294,15 @@ static GtkWidget *map_new(void) {
   osm_gps_map_osd_classic_init(OSM_GPS_MAP(widget));
 
   /* connect to GPS */
-  g_object_set_data(G_OBJECT(widget), "gps_state", 
-		    gps_init(gps_callback, widget));
+  gps_state_t *gps = gps_init();
+  gps_register_callback(gps, gps_callback, widget);
+
+  g_object_set_data(G_OBJECT(widget), "gps_state", gps);
 
   return widget;
 }
 
-#if (MAEMO_VERSION_MAJOR == 5) && !defined(__i386__)
+#if defined(MAEMO5) && !defined(__i386__)
 /* get access to zoom buttons */
 static void
 on_window_realize(GtkWidget *widget, gpointer data) {
@@ -255,13 +322,30 @@ on_window_realize(GtkWidget *widget, gpointer data) {
 #endif
 
 static void on_window_destroy (GtkWidget *widget, gpointer data) {
-  gps_state_t *state = g_object_get_data(G_OBJECT(widget), "gps_state");
-  if(state) gps_release(state);
+#ifdef USE_MAEMO
+  osso_context_t *osso_context =  g_object_get_data(G_OBJECT(widget), "osso-context");
+  g_assert(osso_context);
+
+  if(osso_context)
+    osso_deinitialize(osso_context);
+#endif
 
-  map_save_state(GTK_WIDGET(data));
   gtk_main_quit();
 }
 
+static void on_map_destroy (GtkWidget *widget, gpointer data) {
+  track_save(widget);
+
+  coord_t *last_pos = g_object_get_data(G_OBJECT(widget), "gps_pos"); 
+  if(last_pos) g_free(last_pos);
+
+  gps_state_t *state = g_object_get_data(G_OBJECT(widget), "gps_state");
+  g_assert(state);
+  gps_release(state);
+
+  map_save_state(widget);
+}
+
 int main(int argc, char *argv[]) {
 
   setlocale(LC_ALL, "");
@@ -278,14 +362,17 @@ int main(int argc, char *argv[]) {
 #ifdef USE_MAEMO
   /* Create the hildon program and setup the title */
   HildonProgram *program = HILDON_PROGRAM(hildon_program_get_instance());
-  g_set_application_name("Maep");
+  g_set_application_name("Mæp");
   
   /* Create HildonWindow and set it to HildonProgram */
   GtkWidget *window = hildon_window_new();
   hildon_program_add_window(program, HILDON_WINDOW(window));
 
-#if MAEMO_VERSION_MAJOR == 5
-  gtk_window_set_title(GTK_WINDOW(window), "Maep");
+  g_object_set_data(G_OBJECT(window), "osso-context",
+	    osso_initialize("org.harbaum."APP, VERSION, TRUE, NULL));
+
+#if MAEMO_VERSION_MAJOR >= 5
+  gtk_window_set_title(GTK_WINDOW(window), "Mæp");
 #ifndef __i386__
   g_signal_connect(G_OBJECT(window), "realize", 
 		   G_CALLBACK(on_window_realize), NULL);
@@ -298,15 +385,26 @@ int main(int argc, char *argv[]) {
   GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   /* Set a decent default size for the window. */
   gtk_window_set_default_size(GTK_WINDOW(window), 640, 480);
+
+  gtk_window_set_title(GTK_WINDOW(window), "Mæp");
 #endif
 
+  g_signal_connect(G_OBJECT(window), "destroy", 
+		   G_CALLBACK(on_window_destroy), NULL);
+
   /* create map widget */
   GtkWidget *map = map_new();
 
-  g_signal_connect(G_OBJECT(window), "destroy", 
-		   G_CALLBACK(on_window_destroy), map);
+  /* attach menu to main window, this may create a new container */
+  /* for the main view */
+  GtkWidget *container = menu_create(window, map);
 
-  gtk_container_add(GTK_CONTAINER(window), map);
+  g_signal_connect(G_OBJECT(map), "destroy", 
+		   G_CALLBACK(on_map_destroy), NULL);
+
+  gtk_container_add(GTK_CONTAINER(container), map);
+
+  track_restore(map);
 
   gtk_widget_show_all(GTK_WIDGET(window));
   gtk_main();
